@@ -101,6 +101,20 @@ sub pack ($$%) {
 	return $self;
 }
 
+sub abort_pack ($$) {
+	my $self = shift;
+	my $packer = shift;
+	my $status = shift;
+
+	return $self;
+}
+
+sub has_next_in_chain ($) {
+	my $self = shift;
+
+	return 0;
+}
+
 sub init ($) {
 }
 
@@ -113,17 +127,6 @@ sub set ($%) {
 	$self->{$_} = $values{$_} for keys %values;
 }
 
-sub abort_pack ($$) {
-	my $self = shift;
-	my $packer = shift;
-	my $status = shift;
-
-	$self->set_status($status);
-	$packer->jump('status')->uint32($status)->jump('command-start');
-
-	return $self;
-}
-
 sub to_string ($) {
 	my $self = shift;
 
